#!/usr/bin/env python

import sys

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import rdMolDraw2D
from rdkit.Chem import rdDepictor
rdDepictor.SetPreferCoordGen(True)

import cairosvg

def svgDepict(mol):
    # disable FreeType rendering to enable bold font-weight
    d2d = rdMolDraw2D.MolDraw2DSVG(256,256,-1,-1,True)
    opts = d2d.drawOptions()
    opts.dummiesAreAttachments = True
    opts.explicitMethyl = True
    opts.addStereoAnnotation = True
    opts.useBWAtomPalette()
    
    d2d.DrawMolecule(mol)
    d2d.FinishDrawing()
    svg = d2d.GetDrawingText()
    svg.replace("font-weight:normal", "font-weight:bold")
    svg.replace("font-size:16px", "font-size:20px")
    return svg

# repeat through all the files on the command-line
# we can change this to use the glob module as well
#  e.g., find all the files in a set of folders
embed = AllChem.ETKDGv3()                                                                                                        
embed.useRandomCoords = True
embed.useSmallRingTorsions = True

for argument in sys.argv[1:]:
    # each of these files should have a bunch of SMILES
    with open(argument) as smiles_file:
        for line in smiles_file:
            smiles = line.split()[0]
            name = "_".join(line.split()[1:])
            print("Running", name)
            # add an attachment center if needed
            if not smiles.startswith('*'):
                smiles = '*' + smiles
            mol = Chem.MolFromSmiles(smiles)
            svg = svgDepict(mol)

            # save the SVG
            with open(name+'.svg', 'w') as svg_file:
                svg_file.write(svg)

            # save a PNG
            # cairosvg.svg2png( bytestring=svg, write_to=name+".png" )

            # save an SDF with coordinates
            m2 = Chem.AddHs(mol)
            AllChem.EmbedMolecule(m2, params=embed)
            AllChem.MMFFOptimizeMoleculeConfs(m2, maxIters=2000)
            with open(name + ".mol", 'w') as mol_file:
                mol_file.write(Chem.MolToMolBlock(m2))



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause
"""This script provides .svg and .png ligand template previews of Avogadro."""

import argparse

import cairosvg

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import rdMolDraw2D
from rdkit.Chem import rdDepictor
from rdkit.Chem import rdCIPLabeler

rdDepictor.SetPreferCoordGen(True)


def get_args():
    """Collect command-line arguments"""

    parser = argparse.ArgumentParser(
        description="write .svg and .png ligand template previews for Avogadro",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    parser.add_argument(
        "file",
        help="one or multiple input file(s) to process",
        metavar="FILE",
        nargs="+",
        type=argparse.FileType("rt"),
    )

    parser.add_argument(
        "-a",
        "--atom_palette",
        help="select the atom palette used by RDKit",
        type=str,
        default="bw",
        choices=["avalon", "bw", "cdk", "rdkit"],
    )

    parser.add_argument(
        "-v",
        "--verbose",
        help="""
in addition to records which require your attention, report each record to the CLI""",
        action="store_true",
    )

    parser.add_argument(
        "-p",
        "--png",
        help="complement the generation of previews of .svg by .png files",
        action="store_true",
    )

    return parser.parse_args()


def record_chopper(files_read):
    """Read input file(s) to return a list of records

    A valid record is a line to contain a SMILES string, a label, and
    a name, each separated by one space.  By

    - label`a`, RDKit attempts to provide a preview
    - label `m`, a preview is to be prepared manually (e.g., ChemDraw)
    - label `#` comments out to skip this entry

    The name of a structure is appended by `.svg` and `.png` for each of the
    preview files.  If present, spaces in the name (e.g., `chiral ligand`)
    are converted into an underscore (`chiral_ligand.png`)."""
    temporary_content = []
    for file_read in files_read:
        temporary_content += file_read.readlines()

    record_list = [i.strip() for i in temporary_content if len(i.split()) >= 3]

    return record_list


def select_atom_palette(atom_palette):
    """Adjust the atom palette to be used by RDKit

    This allows to select one of the four atom palettes presented in
    Greg Landrum's blog post by May 26, 2023, or to implicitly use
    RDKit's default.

    https://greglandrum.github.io/rdkit-blog/posts/2023-05-26-drawing-options-explained.html
    """
    choices = {
        "avalon": "opts.useAvalonAtomPalette()",
        "bw": "opts.useBWAtomPalette()",
        "cdk": "opts.useCDKAtomPalette()",
        "rdkit": "",  # i.e. the implicit default
    }
    selection = ""
    selection = choices.get(atom_palette, None)

    return selection if selection is not None else ""


def svgDepict(mol, colors):
    """Define the more general script parameters

    - disable FreeType rendering to enable bold font-weight
    - this is the version for ligands (e.g., show a red metal circle)
    """
    d2d = rdMolDraw2D.MolDraw2DSVG(256, 256, -1, -1, noFreetype=True)
    opts = d2d.drawOptions()
    opts.explicitMethyl = True
    opts.addStereoAnnotation = True
    opts.atomHighlightsAreCircles = True

    # a safeguard if using RDKit's default atom palette
    if colors:
        eval(colors)

    metal_complex = Chem.Mol(mol.ToBinary())
    Chem.Kekulize(metal_complex)
    rdDepictor.Compute2DCoords(metal_complex)
    rdCIPLabeler.AssignCIPLabels(metal_complex)

    # position of the "*" atom
    posStar = 0
    for atom in mol.GetAtoms():
        if atom.GetAtomicNum() == 0:
            posStar = atom.GetIdx()

    d2d.DrawMolecule(metal_complex, highlightAtoms=[posStar])
    d2d.FinishDrawing()
    svg = d2d.GetDrawingText()
    return svg


def is_transition_metal(atom):
    """Define transition metals by their atomic number

    The example of the RDKit Cookbook[1] is extended to assign a dummy
    atom indicated by `*` the atomic number of 0.  To prepare the previews
    about the ligands, this equally is processed as a transition metal."""
    n = atom.GetAtomicNum()
    return (22 <= n <= 29) or (40 <= n <= 47) or (72 <= n <= 79) or (n == 0)


def reset_dative_bonds(mol):
    """Edit some "dative bonds"

    Contrasting to the example in the RDKit Cookbook,[1] and based on a
    discussion in the RDKit user forum[2] about this topic, the previews of
    Avogadro2 use plain single bonds instead of 'dative bonds' (depicted by
    pointy arrows) between ligand(s) and transition metal/dummy atom.

    [1] http://rdkit.org/docs/Cookbook.html#organometallics-with-dative-bonds
    [2] https://github.com/rdkit/rdkit/discussions/6995

    Returns the modified molecule.
    """
    from_atoms = [6, 7, 8, 15, 16]  # i.e., C, N, O, P, S
    pt = Chem.GetPeriodicTable()
    rwmol = Chem.RWMol(mol)
    rwmol.UpdatePropertyCache(strict=False)
    metals = [at for at in rwmol.GetAtoms() if is_transition_metal(at)]
    for metal in metals:
        for nbr in metal.GetNeighbors():
            if (
                nbr.GetAtomicNum() in from_atoms
                and rwmol.GetBondBetweenAtoms(
                    nbr.GetIdx(), metal.GetIdx()
                ).GetBondType()
                == Chem.BondType.SINGLE
            ):
                rwmol.RemoveBond(nbr.GetIdx(), metal.GetIdx())
                rwmol.AddBond(nbr.GetIdx(), metal.GetIdx(), Chem.BondType.SINGLE)
    return rwmol


def generate_previews(line, colors, verbose, png):
    """Provide each record a .svg and .png preview"""
    smiles = line.split()[0]
    name = "_".join(line.split()[2:])

    if verbose:
        print("processing", name)

    mol = Chem.MolFromSmiles(smiles, sanitize=False)
    mol = reset_dative_bonds(mol)
    svg = svgDepict(mol, colors).replace("*", "")

    with open(name + ".svg", "w", encoding="utf-8") as svg_file:
        svg_file.write(svg)

    if png:
        cairosvg.svg2png(bytestring=svg, write_to=name + ".png")


def main():
    """Join the functionalities"""
    args = get_args()
    record_list = record_chopper(args.file)
    colors = select_atom_palette(args.atom_palette)

    process_manually = []
    process_skipped = []

    for record in record_list:
        try:
            split_record = record.split()
            if split_record[1] == "a":
                generate_previews(record, colors, args.verbose, args.png)

            elif split_record[1] == "m":
                process_manually.append(record)

            elif split_record[1] == "#":
                process_skipped.append(record)
            else:
                process_skipped.append(record)
        except OSError as e:
            print(f"error to process {record}, {e}")
        except Exception as e:
            print(f"unexpected error for {record}: {e}")

    if process_manually:
        print("\nentries to be processed manually (label `m`):")
        print(*(entry for entry in process_manually), sep="\n")

    if process_skipped:
        print("\nentries commented out (`#`), or with an unknown label:")
        print(*(entry for entry in process_skipped), sep="\n")


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import os

import requests

from openbabel import pybel
from openbabel import openbabel as ob

# URL for the ideal geometry (PDB)
pdbTemplate = "http://ligand-expo.rcsb.org/reports/{}/{}/{}_ideal.pdb"

# default ligand list
ligands = [
    # 20 common amino acids
    "ALA",
    "ARG",
    "ASP",
    "ASN",
    "CYS",
    "GLU",
    "GLN",
    "GLY",
    "HIS",
    "ILE",
    "LEU",
    "LYS",
    "MET",
    "PHE",
    "PRO",
    "SER",
    "THR",
    "TRP",
    "TYR",
    "VAL",
    # extra
    "PYL",
    "SEC",  # "MLY", "MLZ", "PYL",
    # caps
    "NME",
    "ACE",
    # misc
    "AIB",
]

for ligand in ligands:
    pdb = requests.get(pdbTemplate.format(ligand[0], ligand, ligand))
    with open("temp.pdb", "wb") as handle:
        for block in pdb.iter_content(1024):
            handle.write(block)

    try:
        mol_pdb = next(pybel.readfile("pdb", "temp.pdb"))
    except StopIteration:
        continue

    # fix up some atom orders
    if ligand == "ACE":
        # reorder the atoms to make sure C=O are 3, 4
        # unfortunately, OB won't let H be the first atom
        new_order = [3, 4, 1, 2, 5, 6, 7]
        mol_pdb.OBMol.RenumberAtoms(new_order)
    elif ligand == "SEC":
        new_order = [1, 2, 5, 6, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14]
        mol_pdb.OBMol.RenumberAtoms(new_order)
    elif ligand == "PYL":
        new_order = [18, 14, 15, 17, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 16, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35,36, 37, 38, 39]
        mol_pdb.OBMol.RenumberAtoms(new_order)
    # TODO: MLY, MLZ and others

    # remove the "extra" atoms (H2, OXT, HXT)
    removeAtoms = []
    for atom in mol_pdb.atoms:
      name = atom.OBAtom.GetResidue().GetAtomID(atom.OBAtom)

#      if name.find("HXT") != -1 or name.find("OXT") != -1:
#        removeAtoms.append(atom.OBAtom)
      # NME is tricky
      if ligand == "NME" and name.find("HN2") != -1:
        removeAtoms.append(atom.OBAtom)
      if ligand != "NME" and name.find("H2") != -1:
        removeAtoms.append(atom.OBAtom)
      
    for atom in removeAtoms:
        mol_pdb.OBMol.DeleteAtom(atom, False)

    # convert to a MOPAC internal
    internal = mol_pdb.write("mopin")

    # drop the first two lines of internal
    with open(ligand + ".zmat", "w") as f:
        i = 0
        # drop the first two lines
        for line in internal.splitlines()[2:]:
            items = line.split()
            if len(items) < 10:
                continue

            element = items[0]
            length = items[1]
            angle = items[3]
            torsion = items[5]
            atom1, atom2, atom3 = items[7], items[8], items[9]

            obatom = mol_pdb.atoms[i].OBAtom
            name = obatom.GetResidue().GetAtomID(obatom)
            i += 1

            f.write(
                f"{element:2} {name} {length:2.6} {angle:4.8} {torsion:4.8} {atom1:>3} {atom2:>3} {atom3:>3}\n"
            )

os.remove("temp.pdb")



#!/usr/bin/env python

import sys
import os

from openbabel import openbabel
from openbabel import pybel


# repeat through all the files on the command-line
# we can change this to use the glob module as well
#  e.g., find all the files in a set of folders
for argument in sys.argv[1:]:
    filename, extension = os.path.splitext(argument)

    # read all the molecules from the supplied file
    for mol in pybel.readfile(extension[1:], argument):
        print("Running", mol.title)

        # change the dummy atom to a metal atom
        for atom in mol.atoms:
            if atom.OBAtom.GetAtomicNum() == 0:
                atom.OBAtom.SetAtomicNum(46) # Pd
        
        # generate coordinates
        pybel._builder.Build(mol.OBMol)
        mol.addh()
        
        ff = pybel._forcefields["uff"]
        success = ff.Setup(mol.OBMol)
        if not success:
            sys.exit("Cannot set up forcefield")

        ff.ConjugateGradients(500, 1.0e-4)

        ff.GetCoordinates(mol.OBMol)

        # change the Pd atom back to a dummy atom
        for atom in mol.atoms:
            if atom.OBAtom.GetAtomicNum() == 46:
                atom.OBAtom.SetAtomicNum(0)

        output = mol.title + ".sdf"

        mol.write("sdf", output, overwrite=True)




#!/usr/bin/env python

import sys

import cairosvg

for argument in sys.argv[1:]:

    with open(argument) as f:
        svg = f.read()

    name = argument.split(".")[0]

    # save a PNG
    cairosvg.svg2png( bytestring=svg, scale=3.0, write_to=name+".png" )




#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause
"""Script to flatten CJSON for structure templates of Avogadro2."""

import argparse
import json
from pathlib import Path

# Functions flatten_dumps and minimal contain the pairwise comments
# of `# fmt: off` and `# fmt: on`.  This locally disables an automated
# reformat by Black or Ruff which otherwise would overwrite the functions'
# intentional patterns of indentation, and use of single/double quotes.
# By `# fmt: skip # noqa`,the corresponding line is skipped (in flatten_all).


def recursive_search(path: Path):
    file_list = []
    file_list.extend([x for x in path.iterdir() if x.is_file()])
    for d in [x for x in path.iterdir() if x.is_dir()]:
        file_list.extend(recursive_search(d))
    return file_list


def flatten_arrays(data: dict) -> dict:
    """Turn any lists of simple items (not dicts or lists) into strings."""
    if isinstance(data, list):
        # Turn simple lists into flat strings
        if all(not isinstance(i, (dict, list)) for i in data):
            return json.dumps(data)
        # Recursively flatten any nested lists
        else:
            items = [flatten_arrays(i) for i in data]
            return items
    elif isinstance(data, dict):
        # Recursively flatten all entries
        new = {k: flatten_arrays(v) for k, v in data.items()}
        return new
    else:
        return data


def flatten_dumps(data: dict) -> str:
    """Do the same as json.dumps() but write simple lists on a single line."""
    # fmt: off
    flattened = flatten_arrays(data)
    # Lists are now strings, remove quotes to turn them back into lists
    output = json.dumps(flattened, indent=2).replace('"[', '[').replace(']"', ']')
    # Any strings within lists will have had their quotes escaped, so get rid of escapes
    output = output.replace(r'\"', '"')
    return output
    # fmt: on


def minimal(cjson: dict) -> dict:
    """Reduce a CJSON to core geometry data.

    This retains the atoms with their coordinates, the bonds, and
    the overall charge."""
    # fmt: off
    minimal_cjson = {
        "chemicalJson": cjson.get("chemicalJson", 1),
        "atoms": {
            "coords": {
                "3d": cjson["atoms"]["coords"]["3d"]
            },
            "elements": {
                "number": cjson["atoms"]["elements"]["number"]
            }
        },
        "bonds": {
            "connections": {
                "index": cjson["bonds"]["connections"]["index"]
            },
            "order": cjson["bonds"]["order"]
        }
    }
    # Formal charges are useful but may or may not be there
    if "formalCharges" in cjson["atoms"]:
        minimal_cjson["atoms"]["formalCharges"] = cjson["atoms"]["formalCharges"]
    # Keep total charge if present
    if "properties" in cjson:
        minimal_cjson["properties"] = {}
        for prop in ["totalCharge"]:
            if prop in cjson["properties"]:
                minimal_cjson["properties"][prop] = cjson["properties"][prop]

    return minimal_cjson
    # fmt: on


def round_coords(cjson: dict, places: int) -> dict:
    """Round off the atomic coordinates in a CJSON to a specified number of decimal places."""
    coords = cjson["atoms"]["coords"]["3d"]
    rounded = [round(c, places) for c in coords]
    cjson["atoms"]["coords"]["3d"] = rounded
    return cjson


def flatten_all(
    cjson_list: list[Path],
    dryrun: bool,
    minimize: bool,
    round_coords_places: int,
    validate: bool = False,
):
    """Flatten a list of CJSON files according to the parameters set."""
    checks = {}
    # Read then write each cjson
    for file in cjson_list:
        with open(file, "r", encoding="utf-8") as f:
            cjson = json.load(f)
            if validate:
                print(f"\nread:\n{cjson}")

        if minimize:
            cjson = minimal(cjson)
            if validate:
                print(f"\nminimized:\n{cjson}")

        if round_coords_places:
            cjson = round_coords(cjson, round_coords_places)
            if validate:
                print(f"\nrounded:\n{cjson}")

        flattened = flatten_dumps(cjson)

        if validate:
            print(f"\nflattened:\n{cjson}")
            print(f"\noutput:\n{flattened}")

            # Test we get the same object back as we originally read
            check = (cjson == json.loads(flattened))  # fmt: skip # noqa
            checks[file] = check
            if check is False:
                print(f"{file} was not validated")

        if dryrun:
            print(flattened)
        else:
            with open(file, "w", encoding="utf-8") as f:
                f.write(flattened)


def get_args():
    """Collect command-line arguments"""
    parser = argparse.ArgumentParser(
        description="""
Script to flatten CJSON files of structure templates for Avogadro2""",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "directory",
        help="Directory of files to process",
        type=Path,
    )

    parser.add_argument(
        "-d",
        "--dryrun",
        help="Enable a dryrun mode which does not modify the files",
        action="store_true",
    )

    parser.add_argument(
        "-m",
        "--minimize",
        help="Reduce CJSON to retain only atoms, bonds, and charges",
        action="store_true",
    )

    parser.add_argument(
        "-r",
        "--round_coords",
        metavar="",
        help="""
Define explicitly the number of decimals of atomic coordinates to round
to.  If you choose 0 (zero), the coordinates are not rounded.""",
        type=int,
        default=4,
    )

    parser.add_argument(
        "-v",
        "--validate",
        help="""
Check that the flattened CJSON is parsed to the same data representation
as the original""",
        action="store_true",
    )

    return parser.parse_args()


if __name__ == "__main__":
    args = get_args()
    if args.validate:
        print(args)

    # Get all CJSON files in dir
    file_list = recursive_search(args.directory)
    cjson_list = [f for f in file_list if f.suffix == ".cjson"]

    flatten_all(
        cjson_list, args.dryrun, args.minimize, args.round_coords, args.validate
    )



#!/usr/bin/env python

import sys
import glob

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import rdMolDraw2D
from rdkit.Chem import rdDepictor
rdDepictor.SetPreferCoordGen(True)

import cairosvg

def svgDepict(mol):
    # disable FreeType rendering to enable bold font-weight
    d2d = rdMolDraw2D.MolDraw2DSVG(256,256,-1,-1,noFreetype=True)
    opts = d2d.drawOptions()
    opts.dummiesAreAttachments = True
    opts.explicitMethyl = True
    opts.addStereoAnnotation = True

    mc = Chem.Mol(mol.ToBinary())
    Chem.Kekulize(mc)
    rdDepictor.Compute2DCoords(mc)

    d2d.DrawMolecule(mc)
    d2d.FinishDrawing()
    svg = d2d.GetDrawingText()
    return svg

# repeat through all the files on the command-line
# we can change this to use the glob module as well
#  e.g., find all the files in a set of folders
for filename in glob.iglob("*.smi"):

    smiles = ''
    with open(filename) as f:
        smiles = f.read().strip()
        smiles = '*' + smiles + '*'

    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        print("Error reading SMILES: %s" % smiles, filename)
        continue
    svg = svgDepict(mol)
    svg = svg.replace("font-weight:normal", "font-weight:bold").replace("font-size:16px", "font-size:20px")

    # save the SVG
    name = filename[:-4]
    with open(name+'.svg', 'w') as svg_file:
        svg_file.write(svg)

    # save a PNG
    cairosvg.svg2png(bytestring=svg, scale=2.0, write_to=name+".png")
